import logging
import asyncio

_LOGGER = logging.getLogger(__name__)

class TCPClientHandler:
    def __init__(self, host, port):
        self._host = host
        self._port = port
        self._reader = None
        self._writer = None

    async def open_connection(self):
        """Opens an asynchronous TCP connection."""
        try:
            self._reader, self._writer = await asyncio.open_connection(self._host, self._port)
            _LOGGER.info(f"Connection opened: {self._host}:{self._port}")
        except Exception as e:
            _LOGGER.error(f"Error opening connection: {e}")
            self._reader, self._writer = None, None
            raise

    async def close_connection(self):
        """Closes the TCP connection."""
        if self._writer:
            self._writer.close()
            await self._writer.wait_closed()
            _LOGGER.info("Connection closed.")
        else:
            _LOGGER.warning("No active connection to close.")

    async def send_data(self, data):
        """Sends data over the TCP connection."""
        if self._writer:
            try:
                self._writer.write(data.encode())
                await self._writer.drain()  
                _LOGGER.debug(f"Sent data: {data}")
            except Exception as e:
                _LOGGER.error(f"Error sending data: {e}")
        else:
            _LOGGER.warning("Cannot send data: No active connection.")

    async def read_data(self):
        """Reads data from the TCP connection."""
        if self._reader:
            try:
                data = await self._reader.read(1024)  
                if data:
                    decoded_data = data.decode().strip()
                    _LOGGER.debug(f"Received data: {decoded_data}")
                    return decoded_data
                else:
                    _LOGGER.warning("Connection closed by server.")
                    return None
            except Exception as e:
                _LOGGER.error(f"Error reading data: {e}")
                return None
        else:
            _LOGGER.warning("Cannot read data: No active connection.")
            return None
